precision mediump float;
uniform sampler2D		InputTextureSampler;
varying highp vec2		VSOutTexCoord;

uniform float heightInv;
uniform float widthInv;
uniform int width;
uniform float deblurEnabled;
uniform float weight;
uniform float threshold;
uniform float saturation;
uniform float contrast;
const vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

void main(void)
{
	vec4 outColor;
	vec4 origin = texture2D(InputTextureSampler, VSOutTexCoord);
	outColor = origin;
	
	if (deblurEnabled == 1.0) {
		vec4 imageH = origin
			- (texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2(-widthInv,-heightInv),0.0,1.0)) * 0.08
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2(      0.0,-heightInv),0.0,1.0)) * 0.12
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2( widthInv,-heightInv),0.0,1.0)) * 0.08
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2(-widthInv,       0.0),0.0,1.0)) * 0.12
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2( widthInv,       0.0),0.0,1.0)) * 0.12
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2(-widthInv, heightInv),0.0,1.0)) * 0.08
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2(      0.0, heightInv),0.0,1.0)) * 0.12
			+ texture2D(InputTextureSampler, clamp(VSOutTexCoord + vec2( widthInv, heightInv),0.0,1.0)) * 0.08
			+ origin * 0.2);
		vec4 diff = abs(imageH);
		float minVal = min(diff.r, min(diff.g, diff.b));
		float temp = step(minVal, threshold);
		vec4 imageM = temp * origin + ((1.0 - temp) * clamp(origin + imageH * weight, 0.0, 1.0));
		outColor = imageM;

		if (saturation > 0.0) {
			float luminance = dot(imageM.rgb,luminanceWeighting);
			vec3 greyScaleColor = vec3(luminance);
			vec4 imageS=vec4(mix(greyScaleColor,imageM.rgb,saturation), imageM.a);
			outColor = imageS;
		}
		
		if (contrast > 0.0) {
			outColor = vec4(((outColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), outColor.a);
		}
	}
	gl_FragColor = outColor;
}