precision highp float;
uniform sampler2D InputTextureSampler_0; // bg
uniform sampler2D InputTextureSampler_1; // normal
uniform sampler2D InputTextureSampler_2; // blur1
uniform sampler2D InputTextureSampler_3; // blur3
varying vec2 VSOutTexCoord;

uniform vec3 blurAlpha;
uniform highp float widthOffset;
uniform highp float heightOffset;
uniform highp float sharpen;
uniform  vec4 frequencyRangeValue;
uniform  vec4 frequencyRangeBlur;

void main(){
    vec4 iColor = texture2D(InputTextureSampler_1, VSOutTexCoord);
	vec4 meanColor = texture2D(InputTextureSampler_2, VSOutTexCoord);
	vec4 varColor = texture2D(InputTextureSampler_3, VSOutTexCoord);

	lowp float theta = 0.1;
	mediump float p = clamp((min(iColor.r, meanColor.r - 0.1) - 0.2) * 4.0, 0.0, 1.0);
	mediump float meanVar = (varColor.r + varColor.g + varColor.b) / 3.0;
	mediump float kMin;
	lowp vec3 resultColor;
	lowp float intensity = blurAlpha.r;
	kMin = (1.0 - meanVar / (meanVar + theta)) * p * intensity;
	if(kMin > 1.0 - frequencyRangeValue.x) {
		kMin = kMin * frequencyRangeBlur.x ;	
	} else if (kMin > 1.0 - frequencyRangeValue.y) {
		kMin = kMin * frequencyRangeBlur.y;
	} else if (kMin > 1.0 - frequencyRangeValue.z) {
		kMin = kMin * frequencyRangeBlur.z;
	} else {
		kMin = kMin * frequencyRangeBlur.w;
	}

	resultColor = mix(iColor.rgb, meanColor.rgb, kMin);
	highp float sum = 0.25*iColor.g;
	sum += 0.125*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(-widthOffset,0.0)).g;
	sum += 0.125*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(widthOffset,0.0)).g;
	sum += 0.125*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(0.0,-heightOffset)).g;
	sum += 0.125*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(0.0,heightOffset)).g;
	sum += 0.0625*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(widthOffset,heightOffset)).g;
	sum += 0.0625*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(-widthOffset,-heightOffset)).g;
	sum += 0.0625*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(-widthOffset,heightOffset)).g;
	sum += 0.0625*texture2D(InputTextureSampler_1, VSOutTexCoord+vec2(widthOffset,-heightOffset)).g;

	float hPass = iColor.g-sum+0.5;
	float flag = step(0.5, hPass);
	highp vec3 color = mix(max(vec3(0.0), (2.0*hPass + resultColor - 1.0)), min(vec3(1.0), (resultColor + 2.0*hPass - 1.0)), flag);
	color = mix(resultColor.rgb, color.rgb, sharpen);
	gl_FragColor = vec4(color, 1.0);
}